package com.fjhx.email.service.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.email.config.exception.ServiceException;
import com.fjhx.email.entity.EnterpriseDomain;
import com.fjhx.email.entity.EnterpriseFolder;
import com.fjhx.email.entity.EnterpriseMailbox;
import com.fjhx.email.mapper.EnterpriseMailboxMapper;
import com.fjhx.email.service.IEnterpriseDomainService;
import com.fjhx.email.service.IEnterpriseFolderService;
import com.fjhx.email.service.IEnterpriseMailboxService;
import com.fjhx.email.utils.email.ImapUtil;
import com.sun.mail.imap.IMAPStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.mail.MessagingException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 企业邮箱 服务实现类
 * </p>
 *
 * @author zlj
 * @since 2023-04-06
 */
@Service
public class EnterpriseMailboxServiceImpl extends ServiceImpl<EnterpriseMailboxMapper, EnterpriseMailbox> implements IEnterpriseMailboxService {

    @Autowired
    private IEnterpriseFolderService enterpriseFolderService;

    @Autowired
    private IEnterpriseDomainService enterpriseDomainService;

    @Transactional
    @Override
    public void add(EnterpriseMailbox enterpriseMailbox) {
        Long enterpriseMailboxId = IdWorker.getId();

        Long domainId = enterpriseMailbox.getDomainId();
        EnterpriseDomain enterpriseDomain = enterpriseDomainService.getById(domainId);

        if (enterpriseDomain == null) {
            throw new ServiceException("未找到企业域名");
        }
        IMAPStore store = null;
        try {

            store = ImapUtil.getStore(
                    enterpriseDomain.getReceiveHost(),
                    enterpriseMailbox.getMailUserPrefix() + "@" + enterpriseDomain.getDomainName(),
                    enterpriseMailbox.getMailPassword());

            List<String> folderNameList = ImapUtil.getFolderNameList(store);

            List<EnterpriseFolder> personalFolderList = folderNameList.stream().map(item -> {
                EnterpriseFolder enterpriseFolder = new EnterpriseFolder();
                enterpriseFolder.setMailboxId(enterpriseMailboxId);
                enterpriseFolder.setName(item);
                enterpriseFolder.setSyncStatus(1);
                enterpriseFolder.setDelFlag(0);
                return enterpriseFolder;
            }).collect(Collectors.toList());

            enterpriseFolderService.saveBatch(personalFolderList);

        } catch (MessagingException e) {
            log.error("连接邮箱异常", e);
            throw new ServiceException("连接邮箱异常:" + e.getMessage());
        } finally {
            try {
                ImapUtil.closeStore(store);
            } catch (MessagingException e) {
                log.error("连接关闭失败");
            }
        }

        enterpriseMailbox.setId(enterpriseMailboxId);
        enterpriseMailbox.setDefaultStatus(0);
        enterpriseMailbox.setSyncStatus(1);
        enterpriseMailbox.setStatus(1);
        enterpriseMailbox.setDelFlag(0);
        this.save(enterpriseMailbox);
    }

}
